from django.apps import AppConfig


class WagtailLocalizeTestAppConfig(AppConfig):
    label = "wagtail_localize_test"
    name = "wagtail_localize.test"
    verbose_name = "Wagtail Localize Test app"
