# Generated by Django 3.0.8 on 2020-08-07 10:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_localize_test', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='NonTranslatableSnippet',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('field', models.TextField()),
            ],
        ),
    ]
